From de52c2eefbfb2c9d62d6e5d89263367c9fa12c7c Mon Sep 17 00:00:00 2001
From: Ying Fang <fangying1@huawei.com>
Date: Thu, 31 Dec 2020 16:24:39 +0800
Subject: [PATCH 2/2] pl031: Make read buffer length adaptive

The EDK2 pl031 driver may read the pl031 register byte by byte.
Therefore we must make the read buffer length adaptive otherwise
pl031 may crash at LittleEndian::write_u32.

Signed-off-by: Ying Fang <fangying1@huawei.com>
---
 device_model/src/legacy/pl031.rs | 14 ++++++++++++--
 1 file changed, 12 insertions(+), 2 deletions(-)

diff --git a/device_model/src/legacy/pl031.rs b/device_model/src/legacy/pl031.rs
index 0413c44..3a6a82d 100644
--- a/device_model/src/legacy/pl031.rs
+++ b/device_model/src/legacy/pl031.rs
@@ -95,7 +95,12 @@ impl DeviceOps for PL031 {
     fn read(&mut self, data: &mut [u8], _base: GuestAddress, offset: u64) -> bool {
         if offset >= 0xFE0 && offset < 0x1000 {
             let value = u32::from(RTC_PERIPHERAL_ID[((offset - 0xFE0) >> 2) as usize]);
-            LittleEndian::write_u32(data, value);
+            match data.len() {
+                1 => data[0] = value as u8,
+                2 => LittleEndian::write_u16(data, value as u16),
+                4 => LittleEndian::write_u32(data, value as u32),
+                _ => {}
+            }
             return true;
         }
 
@@ -125,7 +130,12 @@ impl DeviceOps for PL031 {
             _ => {}
         }
 
-        LittleEndian::write_u32(data, value);
+        match data.len() {
+            1 => data[0] = value as u8,
+            2 => LittleEndian::write_u16(data, value as u16),
+            4 => LittleEndian::write_u32(data, value as u32),
+            _ => {}
+        }
 
         true
     }
-- 
2.27.0

